import { geoPath } from "d3-geo";
import { geoArea } from "d3-geo";
import { bbox } from "../bbox.js";
import { getproj } from "../projections/projections.js";
const d3 = Object.assign({}, { geoPath, geoArea });

export function minimap({
  mainmap = {
    selection: null,
    projection: null,
    width: null,
    height: null,
  },
  id,
  x = 5,
  y = 5,
  geojson = land,
  extent = null,
  threshold = 0.1,
  precision = 10,
  projection = "Equirectangular",
  width = 200,
  background = {
    stroke: "black",
    strokeWidth: 1,
    strokeOpacity: 1,
    fill: "white",
    fillOpacity: 1,
  },
  frame = {
    stroke: "none",
    strokeWidth: 1,
    strokeOpacity: 1,
    fill: "red",
    fillOpacity: 0.25,
  },
  geometries = {
    fill: "#CCC",
    stroke: "none",
    strokeWidth: 1,
    fillOpacity: 1,
    strokeOpacity: 1,
  },
  raise = {
    fill: "red",
    stroke: "none",
    strokeWidth: 1,
    fillOpacity: 1,
    strokeOpacity: 1,
  },
  dot = {
    fill: "red",
    r: 5,
    stroke: "none",
    strokeWidth: 1,
    fillOpacity: 1,
    strokeOpacity: 1,
  },
} = {}) {
  // inique id
  const clipid =
    Date.now().toString(36) + Math.random().toString(36).substring(2);

  // extent

  extent =
    Array.isArray(extent) &&
    Array.isArray(extent[0]) &&
    Array.isArray(extent[1])
      ? bbox(extent)
      : extent;

  const outline = { type: "Sphere" };

  // selection
  let selection = mainmap.selection;

  let sel = selection
    .append("g")
    .attr("class", id)
    .attr("data-layer", JSON.stringify({ _type: "minimap" }));

  // projection
  projection = getproj(projection);
  projection = projection.fitWidth(width, extent ? extent : outline);
  const path = d3.geoPath(projection);

  // extent clip
  let bb = "";
  if (extent != null) {
    bb = path.bounds(extent);
    sel
      .append("clipPath")
      .attr("id", `extent_${clipid}`)
      .append("rect")
      .attr("x", 0)
      .attr("y", 0)
      .attr("width", bb[1][0])
      .attr("height", bb[1][1]);
  }

  // Outline (fill)

  if (extent == null) {
    sel
      .append("g")
      .append("path")
      .attr("d", path(outline))
      .attr("fill", background.fill)
      .attr("fill-opacity", background.fillOpacity)
      .attr("stroke", "none")
      .attr("transform", `translate(${x} ${y})`);
  } else {
    sel
      .append("g")
      .append("rect")
      .attr("x", 0)
      .attr("y", 0)
      .attr("width", bb[1][0])
      .attr("height", bb[1][1])
      .attr("fill", background.fill)
      .attr("fill-opacity", background.fillOpacity)
      .attr("stroke", "none")
      .attr("transform", `translate(${x} ${y})`);
  }

  // world
  sel
    .append("g")
    .append("path")
    .datum(geojson)
    .attr("fill", geometries.fill)
    .attr("fill-opacity", geometries.fillOpacity)
    .attr("stroke", geometries.stroke)
    .attr("stroke-width", geometries.strokeWidth)
    .attr("stroke-opacity", geometries.strokeOpacity)
    .attr("d", path)
    .attr("transform", `translate(${x} ${y})`)
    .attr("clip-path", `url(#extent_${clipid})`); // ICI

  // Outline (stroke)

  if (extent == null) {
    sel
      .append("g")
      .append("path")
      .attr("d", path(outline))
      .attr("fill", "none")
      .attr("stroke", background.stroke)
      .attr("stroke-width", background.strokeWidth)
      .attr("stroke-opacity", background.strokeOpacity)
      .attr("transform", `translate(${x} ${y})`);
  } else {
    sel
      .append("g")
      .append("rect")
      .attr("x", 0)
      .attr("y", 0)
      .attr("width", bb[1][0])
      .attr("height", bb[1][1])
      .attr("fill", "none")
      .attr("stroke", background.stroke)
      .attr("stroke-width", background.strokeWidth)
      .attr("stroke-opacity", background.strokeOpacity)
      .attr("transform", `translate(${x} ${y})`);
  }

  // Map area
  let rangx = [];
  for (let i = 0; i <= mainmap.width; i += mainmap.width / precision) {
    rangx.push(i);
  }
  let rangy = [];
  for (let i = 0; i <= mainmap.height; i += mainmap.width / precision) {
    rangy.push(i);
  }

  let coords = [];
  coords.push(rangx.map((d) => mainmap.projection.invert([d, 0])));
  coords.push(rangy.map((d) => mainmap.projection.invert([mainmap.width, d])));
  coords.push(
    rangx.reverse().map((d) => mainmap.projection.invert([d, mainmap.height]))
  );
  coords.push(rangy.reverse().map((d) => mainmap.projection.invert([0, d])));

  const rect = {
    type: "Feature",
    geometry: {
      type: "Polygon",
      coordinates: [coords.flat()],
    },
  };

  if (d3.geoArea(rect) > threshold) {
    sel
      .append("g")
      .append("path")
      .datum(rect)
      .attr("fill", frame.fill)
      .attr("fill-opacity", frame.fillOpacity)
      .attr("stroke", frame.stroke)
      .attr("stroke-width", frame.strokeWidth)
      .attr("stroke-opacity", frame.strokeOpacity)
      .attr("d", path)
      .attr("transform", `translate(${x} ${y})`);

    // clip

    sel
      .append("clipPath")
      .attr("id", `clipminimap_${clipid}`)
      .append("path")
      .datum(rect)
      .attr("d", d3.geoPath(projection));

    sel
      .append("g")
      .append("path")
      .attr("clip-path", `url(#clipminimap_${clipid})`)
      .datum(geojson)
      .attr("fill", raise.fill)
      .attr("fill-opacity", raise.fillOpacity)
      .attr("stroke", raise.stroke)
      .attr("stroke-width", raise.strokeWidth)
      .attr("stroke-opacity", raise.strokeOpacity)
      .attr("d", path)
      .attr("transform", `translate(${x} ${y})`);
  } else {
    const center = mainmap.projection.invert([
      mainmap.width / 2,
      mainmap.height / 2,
    ]);

    sel
      .append("circle")
      .attr("cx", projection(center)[0])
      .attr("cy", projection(center)[1])
      .attr("fill", dot.fill)
      .attr("fill-opacity", dot.fillOpacity)
      .attr("stroke", dot.stroke)
      .attr("stroke-width", dot.strokeWidth)
      .attr("stroke-opacity", dot.strokeOpacity)
      .attr("r", dot.r)
      .attr("transform", `translate(${x} ${y})`);
  }
}

// Land ----------------------------------------------------------------

const land = {
  type: "FeatureCollection",
  features: [
    {
      type: "Feature",
      properties: {},
      geometry: {
        type: "MultiPolygon",
        coordinates: [
          [
            [
              [9.8, 1],
              [4.41, 6.36],
              [-7.54, 4.35],
              [-16.73, 12.33],
              [-17.01, 21.2],
              [-5.91, 35.81],
              [11.31, 37],
              [10.33, 33.7],
              [19.06, 30.27],
              [21.61, 32.93],
              [34.48, 31.58],
              [36.02, 36.93],
              [27.67, 36.66],
              [26.74, 40.4],
              [41.5, 42.64],
              [31.19, 46.63],
              [22.57, 40.05],
              [13.71, 45.59],
              [15.61, 40.07],
              [3.94, 43.53],
              [-2.13, 36.74],
              [-9, 37.02],
              [-9.29, 42.92],
              [-1.79, 43.39],
              [-1.37, 48.66],
              [9.44, 54.81],
              [21.04, 55.3],
              [21.85, 65.54],
              [13.28, 55.35],
              [7.42, 62.6],
              [18.6, 70.1],
              [27.77, 71.19],
              [41.02, 67.7],
              [34.77, 64.56],
              [66.93, 71.3],
              [102.03, 77.22],
              [111.14, 76.75],
              [106.36, 73.19],
              [124.38, 73.81],
              [130.57, 70.96],
              [141.55, 72.77],
              [160.54, 69.36],
              [179.95, 68.97],
              [179.61, 62.71],
              [163.39, 59.62],
              [156.72, 50.89],
              [155.55, 55.28],
              [164.03, 61.35],
              [146.33, 59.71],
              [135.22, 54.89],
              [141.53, 53.18],
              [140.17, 48.45],
              [126.67, 37.83],
              [117.76, 39.11],
              [121.63, 28.54],
              [116.49, 22.94],
              [106.79, 21.01],
              [109.44, 14.1],
              [104.89, 9.85],
              [91.62, 22.54],
              [80.26, 15.67],
              [76.54, 8.94],
              [70.98, 20.71],
              [66.25, 25.47],
              [57.33, 25.78],
              [47.95, 29.99],
              [51.22, 24.63],
              [56.08, 26.11],
              [57.84, 19.02],
              [43.95, 12.6],
              [35.22, 28.05],
              [39.72, 15.26],
              [44.27, 10.46],
              [51.29, 11.83],
              [47.95, 4.46],
              [40.17, -2.76],
              [39.86, -16.43],
              [27.1, -33.53],
              [19.95, -34.81],
              [11.85, -18.14],
              [13.64, -12.25],
              [9.8, 1],
            ],
            [
              [27.64, 58],
              [27.64, 58],
            ],
            [
              [5.07, 52.92],
              [5.07, 52.92],
            ],
            [
              [133.26, 45.54],
              [133.26, 45.54],
            ],
            [
              [57.64, 42.15],
              [57.64, 42.15],
            ],
            [
              [60.06, 44.85],
              [60.06, 44.85],
            ],
            [
              [34.86, -13.51],
              [34.86, -13.51],
            ],
            [
              [29.21, -3.33],
              [29.21, -3.33],
            ],
            [
              [34, 0.25],
              [34, 0.25],
            ],
            [
              [48.87, 38.43],
              [54.01, 36.82],
              [50.3, 44.66],
              [48.87, 38.43],
            ],
          ],
          [
            [
              [8.72, 3.8],
              [8.72, 3.8],
            ],
          ],
          [
            [
              [179.08, -8.36],
              [179.08, -8.36],
            ],
          ],
          [
            [
              [179.18, -8],
              [179.18, -8],
            ],
          ],
          [
            [
              [73.71, 4.23],
              [73.71, 4.23],
            ],
          ],
          [
            [
              [73.41, 5.19],
              [73.41, 5.19],
            ],
          ],
          [
            [
              [73.37, 3.53],
              [73.37, 3.53],
            ],
          ],
          [
            [
              [-61.77, 17.13],
              [-61.77, 17.13],
            ],
          ],
          [
            [
              [50.42, 26.6],
              [50.42, 26.6],
            ],
          ],
          [
            [
              [-78.15, 25.26],
              [-78.15, 25.26],
            ],
          ],
          [
            [
              [-77.25, 25.13],
              [-77.25, 25.13],
            ],
          ],
          [
            [
              [-59.53, 13.06],
              [-59.53, 13.06],
            ],
          ],
          [
            [
              [43.87, -12.2],
              [43.87, -12.2],
            ],
          ],
          [
            [
              [-23.9, 15.8],
              [-23.9, 15.8],
            ],
          ],
          [
            [
              [-61.45, 15.56],
              [-61.45, 15.56],
            ],
          ],
          [
            [
              [11.95, 55.99],
              [11.95, 55.99],
            ],
          ],
          [
            [
              [-16.13, 28.6],
              [-16.13, 28.6],
            ],
          ],
          [
            [
              [3.2, 39.88],
              [3.2, 39.88],
            ],
          ],
          [
            [
              [-7.41, 62.14],
              [-7.41, 62.14],
            ],
          ],
          [
            [
              [158.06, 6.98],
              [158.06, 6.98],
            ],
          ],
          [
            [
              [-2.51, 49.41],
              [-2.51, 49.41],
            ],
          ],
          [
            [
              [-61.56, 12.03],
              [-61.56, 12.03],
            ],
          ],
          [
            [
              [-4.6, 54.09],
              [-4.6, 54.09],
            ],
          ],
          [
            [
              [15.66, 38.42],
              [15.66, 38.42],
            ],
          ],
          [
            [
              [9.12, 41.24],
              [9.12, 41.24],
            ],
          ],
          [
            [
              [-2.12, 49.26],
              [-2.12, 49.26],
            ],
          ],
          [
            [
              [-158.96, 2.2],
              [-158.96, 2.2],
            ],
          ],
          [
            [
              [-62.67, 17.34],
              [-62.67, 17.34],
            ],
          ],
          [
            [
              [-60.95, 13.71],
              [-60.95, 13.71],
            ],
          ],
          [
            [
              [171.05, 7.14],
              [171.05, 7.14],
            ],
          ],
          [
            [
              [14.55, 35.76],
              [14.55, 35.76],
            ],
          ],
          [
            [
              [57.71, -20.1],
              [57.71, -20.1],
            ],
          ],
          [
            [
              [167.02, -0.55],
              [167.02, -0.55],
            ],
          ],
          [
            [
              [134.66, 7.59],
              [134.66, 7.59],
            ],
          ],
          [
            [
              [104.28, 1.37],
              [104.28, 1.37],
            ],
          ],
          [
            [
              [6.69, 0.07],
              [6.69, 0.07],
            ],
          ],
          [
            [
              [55.49, -4.52],
              [55.49, -4.52],
            ],
          ],
          [
            [
              [-175.25, -20.99],
              [-175.25, -20.99],
            ],
          ],
          [
            [
              [-61.2, 13.1],
              [-61.2, 13.1],
            ],
          ],
          [
            [
              [-172.27, -13.44],
              [-172.27, -13.44],
            ],
          ],
          [
            [
              [-53.38, -33.74],
              [-64.89, -40.71],
              [-65.97, -48.05],
              [-71.03, -53.81],
              [-75.43, -47.25],
              [-72.46, -41.99],
              [-70.39, -18.34],
              [-81.1, -6.07],
              [-77.34, 6.54],
              [-94.85, 16.43],
              [-103.5, 18.33],
              [-123.73, 38.92],
              [-122.21, 47.99],
              [-134.24, 58.21],
              [-151.57, 60.99],
              [-156.55, 56.98],
              [-166.37, 63.1],
              [-166.55, 68.36],
              [-156.82, 71.31],
              [-135.87, 68.91],
              [-127.84, 70.24],
              [-113.86, 67.7],
              [-96.46, 67.47],
              [-96.88, 71.01],
              [-81.24, 67.46],
              [-92.45, 62.79],
              [-92.42, 57.35],
              [-79.77, 54.65],
              [-77.49, 62.59],
              [-67.03, 58.18],
              [-64.07, 60.45],
              [-55.81, 53.33],
              [-66.41, 50.26],
              [-64.51, 46.24],
              [-73.65, 41],
              [-76.07, 35.38],
              [-84.02, 30.11],
              [-97.55, 27.01],
              [-95.75, 18.77],
              [-86.83, 21.43],
              [-88.69, 15.84],
              [-83.13, 15],
              [-82.25, 9.01],
              [-62.62, 9.74],
              [-52.99, 5.46],
              [-50.4, -0.07],
              [-35.51, -5.14],
              [-42.06, -22.95],
              [-48.58, -25.87],
              [-53.38, -33.74],
            ],
            [
              [-79.06, 43.28],
              [-79.06, 43.28],
            ],
            [
              [-82.64, 42.56],
              [-82.64, 42.56],
            ],
            [
              [-89.58, 48],
              [-89.58, 48],
            ],
            [
              [-69.37, -15.52],
              [-69.37, -15.52],
            ],
            [
              [-84.11, 46.35],
              [-84.11, 46.35],
            ],
          ],
          [
            [
              [9.44, 42.77],
              [9.44, 42.77],
            ],
          ],
          [
            [
              [-61.9, 16.22],
              [-61.9, 16.22],
            ],
          ],
          [
            [
              [-60.6, 14.51],
              [-60.6, 14.51],
            ],
          ],
          [
            [
              [55.75, -20.58],
              [55.75, -20.58],
            ],
          ],
          [
            [
              [143.1, 54.4],
              [143.1, 54.4],
            ],
          ],
          [
            [
              [-179.96, 65.13],
              [-179.94, 68.93],
              [-170.01, 66.09],
              [-179.96, 65.13],
            ],
          ],
          [
            [
              [50.59, 69.33],
              [50.59, 69.33],
            ],
          ],
          [
            [
              [-179.92, 70.99],
              [-179.92, 70.99],
            ],
          ],
          [
            [
              [143.02, 74.29],
              [143.02, 74.29],
            ],
          ],
          [
            [
              [146.73, 75.49],
              [146.73, 75.49],
            ],
          ],
          [
            [
              [139.11, 76.1],
              [139.11, 76.1],
            ],
          ],
          [
            [
              [67.15, 76.1],
              [67.15, 76.1],
            ],
          ],
          [
            [
              [102.93, 79.28],
              [102.93, 79.28],
            ],
          ],
          [
            [
              [97.77, 80.13],
              [97.77, 80.13],
            ],
          ],
          [
            [
              [48.83, 80.81],
              [48.83, 80.81],
            ],
          ],
          [
            [
              [96.49, 81.64],
              [96.49, 81.64],
            ],
          ],
          [
            [
              [-68.63, -52.64],
              [-68.63, -52.64],
            ],
          ],
          [
            [
              [180, -90],
              [-180, -90],
              [-180, -84.35],
              [-157.56, -83.39],
              [-149.08, -79.48],
              [-157.93, -77.06],
              [-133.87, -74.93],
              [-98.4, -75.24],
              [-102.72, -73],
              [-77.24, -73.81],
              [-67.51, -72.83],
              [-64.78, -67.02],
              [-59.9, -72.99],
              [-63.16, -75.41],
              [-80.3, -78.85],
              [-61.85, -83.37],
              [-29.76, -80.18],
              [-33.72, -77.31],
              [-18.87, -75.25],
              [-9.45, -71.1],
              [26.45, -71.03],
              [50.32, -66.39],
              [69.56, -67.74],
              [67, -72.9],
              [84.96, -67.08],
              [113.2, -65.77],
              [130.73, -66.15],
              [167.7, -70.79],
              [162.19, -75.39],
              [166.09, -78.57],
              [160.13, -81.2],
              [180, -84.35],
              [180, -90],
            ],
          ],
          [
            [
              [-60.07, -79.69],
              [-60.07, -79.69],
            ],
          ],
          [
            [
              [-160.29, -79.27],
              [-160.29, -79.27],
            ],
          ],
          [
            [
              [-66.83, -78.44],
              [-66.83, -78.44],
            ],
          ],
          [
            [
              [-45.35, -78.69],
              [-45.35, -78.69],
            ],
          ],
          [
            [
              [-121.2, -73.5],
              [-121.2, -73.5],
            ],
          ],
          [
            [
              [-126.08, -73.3],
              [-126.08, -73.3],
            ],
          ],
          [
            [
              [-73.93, -72.46],
              [-73.93, -72.46],
            ],
          ],
          [
            [
              [-96.74, -71.63],
              [-96.74, -71.63],
            ],
          ],
          [
            [
              [-70.02, -69.19],
              [-70.02, -69.19],
            ],
          ],
          [
            [
              [144.4, -40.46],
              [144.4, -40.46],
            ],
          ],
          [
            [
              [137.56, -35.64],
              [137.56, -35.64],
            ],
          ],
          [
            [
              [130.24, -11.09],
              [130.24, -11.09],
            ],
          ],
          [
            [
              [142.55, -10.71],
              [146.28, -18.87],
              [153.21, -25.93],
              [150.01, -37.7],
              [140.36, -37.88],
              [131.18, -31.48],
              [115.12, -34.36],
              [113.65, -22.58],
              [130.99, -12.24],
              [140, -17.71],
              [142.55, -10.71],
            ],
          ],
          [
            [
              [109.65, 2.08],
              [116.78, 7.03],
              [115.97, -3.61],
              [109.65, 2.08],
            ],
          ],
          [
            [
              [-66.84, -55.18],
              [-66.84, -55.18],
            ],
          ],
          [
            [
              [-73.17, -53.52],
              [-73.17, -53.52],
            ],
          ],
          [
            [
              [-74.68, -50],
              [-74.68, -50],
            ],
          ],
          [
            [
              [-73.89, -42.06],
              [-73.89, -42.06],
            ],
          ],
          [
            [
              [-75.1, 19.9],
              [-75.1, 19.9],
            ],
          ],
          [
            [
              [33.02, 34.57],
              [33.02, 34.57],
            ],
          ],
          [
            [
              [-71.76, 19.71],
              [-71.76, 19.71],
            ],
          ],
          [
            [
              [179.99, -16.17],
              [179.99, -16.17],
            ],
          ],
          [
            [
              [178.27, -17.35],
              [178.27, -17.35],
            ],
          ],
          [
            [
              [-59.2, -51.32],
              [-59.2, -51.32],
            ],
          ],
          [
            [
              [-58.78, -51.32],
              [-58.78, -51.32],
            ],
          ],
          [
            [
              [-7.25, 55.07],
              [-7.25, 55.07],
            ],
          ],
          [
            [
              [-3.35, 58.65],
              [1.36, 51.13],
              [-5.44, 50.19],
              [-3.35, 58.65],
            ],
          ],
          [
            [
              [23.9, 35.53],
              [23.9, 35.53],
            ],
          ],
          [
            [
              [23.39, 39.02],
              [23.39, 39.02],
            ],
          ],
          [
            [
              [-52.73, 69.93],
              [-52.73, 69.93],
            ],
          ],
          [
            [
              [-25.45, 70.62],
              [-25.45, 70.62],
            ],
          ],
          [
            [
              [-32.21, 83.61],
              [-13.54, 81.21],
              [-19.08, 80.28],
              [-22.37, 70.12],
              [-41.06, 65.1],
              [-43.14, 60.08],
              [-48.64, 61.24],
              [-59.82, 75.91],
              [-72.59, 78.52],
              [-61, 81.49],
              [-32.21, 83.61],
            ],
          ],
          [
            [
              [120.01, -9.36],
              [120.01, -9.36],
            ],
          ],
          [
            [
              [124.47, -8.97],
              [124.47, -8.97],
            ],
          ],
          [
            [
              [116.82, -8.31],
              [116.82, -8.31],
            ],
          ],
          [
            [
              [117.97, -8.1],
              [117.97, -8.1],
            ],
          ],
          [
            [
              [124.04, -7.96],
              [124.04, -7.96],
            ],
          ],
          [
            [
              [115.49, -8.15],
              [115.49, -8.15],
            ],
          ],
          [
            [
              [138.52, -8.15],
              [138.52, -8.15],
            ],
          ],
          [
            [
              [115, -6.89],
              [115, -6.89],
            ],
          ],
          [
            [
              [106.16, -6.01],
              [106.16, -6.01],
            ],
          ],
          [
            [
              [134.58, -5.42],
              [134.58, -5.42],
            ],
          ],
          [
            [
              [127.07, -3.18],
              [127.07, -3.18],
            ],
          ],
          [
            [
              [129.1, -2.86],
              [129.1, -2.86],
            ],
          ],
          [
            [
              [107.85, -2.55],
              [107.85, -2.55],
            ],
          ],
          [
            [
              [124.53, -1.64],
              [124.53, -1.64],
            ],
          ],
          [
            [
              [135.48, -1.59],
              [135.48, -1.59],
            ],
          ],
          [
            [
              [105.88, -1.49],
              [105.88, -1.49],
            ],
          ],
          [
            [
              [98.86, -0.79],
              [98.86, -0.79],
            ],
          ],
          [
            [
              [140.98, -9.11],
              [133.89, -3.81],
              [137.79, -1.48],
              [147.45, -5.96],
              [140.98, -9.11],
            ],
          ],
          [
            [
              [97.41, 1.72],
              [97.41, 1.72],
            ],
          ],
          [
            [
              [125.06, 1.67],
              [125.06, 1.67],
            ],
          ],
          [
            [
              [128.17, 1.92],
              [128.17, 1.92],
            ],
          ],
          [
            [
              [95.5, 5.61],
              [103.76, 0.24],
              [104.54, -5.84],
              [95.5, 5.61],
            ],
          ],
          [
            [
              [92.79, 13.31],
              [92.79, 13.31],
            ],
          ],
          [
            [
              [-16.04, 66.54],
              [-16.04, 66.54],
            ],
          ],
          [
            [
              [-76.7, 18.45],
              [-76.7, 18.45],
            ],
          ],
          [
            [
              [130.98, 33.79],
              [130.98, 33.79],
            ],
          ],
          [
            [
              [134.16, 34.36],
              [134.16, 34.36],
            ],
          ],
          [
            [
              [141.28, 41.35],
              [141.28, 41.35],
            ],
          ],
          [
            [
              [142.02, 45.46],
              [142.02, 45.46],
            ],
          ],
          [
            [
              [80.41, 9.71],
              [80.41, 9.71],
            ],
          ],
          [
            [
              [49.35, -12.09],
              [47.13, -24.93],
              [43.94, -17.46],
              [49.35, -12.09],
            ],
          ],
          [
            [
              [22.85, 78.36],
              [22.85, 78.36],
            ],
          ],
          [
            [
              [16.82, 79.88],
              [16.82, 79.88],
            ],
          ],
          [
            [
              [23.14, 80.38],
              [23.14, 80.38],
            ],
          ],
          [
            [
              [172.9, -40.51],
              [172.9, -40.51],
            ],
          ],
          [
            [
              [173.08, -34.41],
              [173.08, -34.41],
            ],
          ],
          [
            [
              [125.64, 9.61],
              [125.64, 9.61],
            ],
          ],
          [
            [
              [123.27, 10.98],
              [123.27, 10.98],
            ],
          ],
          [
            [
              [124, 11.08],
              [124, 11.08],
            ],
          ],
          [
            [
              [119.82, 11.47],
              [119.82, 11.47],
            ],
          ],
          [
            [
              [124.44, 11.45],
              [124.44, 11.45],
            ],
          ],
          [
            [
              [122.3, 11.76],
              [122.3, 11.76],
            ],
          ],
          [
            [
              [124.92, 12.56],
              [124.92, 12.56],
            ],
          ],
          [
            [
              [121.17, 13.38],
              [121.17, 13.38],
            ],
          ],
          [
            [
              [120.91, 18.57],
              [120.91, 18.57],
            ],
          ],
          [
            [
              [154.81, -5.48],
              [154.81, -5.48],
            ],
          ],
          [
            [
              [152.23, -4.19],
              [152.23, -4.19],
            ],
          ],
          [
            [
              [151.04, -2.71],
              [151.04, -2.71],
            ],
          ],
          [
            [
              [-67.11, 18.55],
              [-67.11, 18.55],
            ],
          ],
          [
            [
              [161.69, -10.04],
              [161.69, -10.04],
            ],
          ],
          [
            [
              [159.62, -9.38],
              [159.62, -9.38],
            ],
          ],
          [
            [
              [160.8, -8.36],
              [160.8, -8.36],
            ],
          ],
          [
            [
              [158.6, -7.59],
              [158.6, -7.59],
            ],
          ],
          [
            [
              [-60.22, 10.8],
              [-60.22, 10.8],
            ],
          ],
          [
            [
              [121.64, 25.22],
              [121.64, 25.22],
            ],
          ],
          [
            [
              [-155.73, 20.19],
              [-155.73, 20.19],
            ],
          ],
          [
            [
              [-163.76, 55.06],
              [-163.76, 55.06],
            ],
          ],
          [
            [
              [-133.53, 56.59],
              [-133.53, 56.59],
            ],
          ],
          [
            [
              [-135.67, 57.36],
              [-135.67, 57.36],
            ],
          ],
          [
            [
              [-153.01, 57.93],
              [-153.01, 57.93],
            ],
          ],
          [
            [
              [-135.71, 58.23],
              [-135.71, 58.23],
            ],
          ],
          [
            [
              [-135.43, 58.33],
              [-135.43, 58.33],
            ],
          ],
          [
            [
              [-166.09, 60.46],
              [-166.09, 60.46],
            ],
          ],
          [
            [
              [-171.67, 63.8],
              [-171.67, 63.8],
            ],
          ],
          [
            [
              [166.95, -15.02],
              [166.95, -15.02],
            ],
          ],
          [
            [
              [-83.84, 46.15],
              [-83.84, 46.15],
            ],
          ],
          [
            [
              [-64.34, 47.05],
              [-64.34, 47.05],
            ],
          ],
          [
            [
              [-60.5, 47],
              [-60.5, 47],
            ],
          ],
          [
            [
              [-64.13, 49.95],
              [-64.13, 49.95],
            ],
          ],
          [
            [
              [-55.7, 51.32],
              [-55.7, 51.32],
            ],
          ],
          [
            [
              [-81.43, 53.37],
              [-81.43, 53.37],
            ],
          ],
          [
            [
              [-132.69, 53.93],
              [-132.69, 53.93],
            ],
          ],
          [
            [
              [-81.74, 63.06],
              [-81.74, 63.06],
            ],
          ],
          [
            [
              [-85.49, 65.8],
              [-85.49, 65.8],
            ],
          ],
          [
            [
              [-75.44, 68.71],
              [-75.44, 68.71],
            ],
          ],
          [
            [
              [-99.03, 70.19],
              [-99.03, 70.19],
            ],
          ],
          [
            [
              [-114.2, 73.31],
              [-104.87, 73.13],
              [-101.81, 69.01],
              [-113.35, 68.59],
              [-114.2, 73.31],
            ],
          ],
          [
            [
              [-81.06, 73.85],
              [-81.06, 73.85],
            ],
          ],
          [
            [
              [-86.56, 73.86],
              [-68.36, 70.58],
              [-61.89, 66.88],
              [-64.54, 61.74],
              [-73.2, 64.22],
              [-72.99, 68.24],
              [-88.69, 70.47],
              [-86.56, 73.86],
            ],
          ],
          [
            [
              [-99.83, 73.9],
              [-99.83, 73.9],
            ],
          ],
          [
            [
              [-92.75, 74.09],
              [-92.75, 74.09],
            ],
          ],
          [
            [
              [-121.58, 74.56],
              [-121.58, 74.56],
            ],
          ],
          [
            [
              [-94.39, 75.6],
              [-94.39, 75.6],
            ],
          ],
          [
            [
              [-98.41, 76.67],
              [-98.41, 76.67],
            ],
          ],
          [
            [
              [-108.67, 76.82],
              [-108.67, 76.82],
            ],
          ],
          [
            [
              [-95.24, 77.01],
              [-79.65, 75.47],
              [-89.95, 74.53],
              [-95.24, 77.01],
            ],
          ],
          [
            [
              [-116.13, 77.47],
              [-116.13, 77.47],
            ],
          ],
          [
            [
              [-109.6, 78.07],
              [-109.6, 78.07],
            ],
          ],
          [
            [
              [-96.23, 78.64],
              [-96.23, 78.64],
            ],
          ],
          [
            [
              [-111.36, 79.26],
              [-111.36, 79.26],
            ],
          ],
          [
            [
              [-103.66, 79.34],
              [-103.66, 79.34],
            ],
          ],
          [
            [
              [-94.75, 82.12],
              [-94.75, 82.12],
            ],
          ],
          [
            [
              [-69.69, 83.12],
              [-61.31, 82.2],
              [-77.77, 76.66],
              [-86.91, 77.27],
              [-90.81, 81.72],
              [-69.69, 83.12],
            ],
          ],
          [
            [
              [-127.81, 50.44],
              [-127.81, 50.44],
            ],
          ],
          [
            [
              [163.52, -20.03],
              [163.52, -20.03],
            ],
          ],
          [
            [
              [110.69, 20.15],
              [110.69, 20.15],
            ],
          ],
        ],
      },
    },
  ],
};
